/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import jreqtools.gadgets.*;

/**
* This is a requester for colors.
* 
* @author	Markus Hillenbrand
*/

public class ColorRequester extends AbstractRequester {
	private Color 			color 		= null;
	private	int				tabindex	= 0;
	private	int 				answer		= CANCEL;
	
	private class Dialog extends JDialog {
		private	JPanel			colorfield		= new JPanel();
		private	JSlider			hueslider		= new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
		private	JSlider			saturationslider	= new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
		private	JSlider			brightnessslider	= new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
		private	JSlider			redslider			= new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
		private	JSlider			greenslider		= new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
		private	JSlider			blueslider		= new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
		private	TextField		redtext			= new TextField("255");
		private	TextField		greentext		= new TextField("255");
		private	TextField		bluetext			= new TextField("25");
		private	TextField		huetext			= new TextField("100");
		private	TextField		saturationtext		= new TextField("100");
		private	TextField		brightnesstext	= new TextField("100");
		private JTabbedPane	tabbedpane			= new JTabbedPane();
		private	boolean			handleEvent		= true;
		
		public Dialog(JFrame parent)  {
			super(parent, "JReqTools: ColorRequester", true);
			
			// TabbedPane erzeugen
			tabbedpane.addTab(JReqTools.getResourceString("RGBModel"), createRGBPane());
			tabbedpane.addTab(JReqTools.getResourceString("HSBModel"), createHSBPane());
			
			// RGB Werte holen
			int red = color.getRed();
			int green = color.getGreen();
			int blue = color.getBlue();
			
			// HSB Werte ausrechnen
			float [] hsb = Color.RGBtoHSB(red, green, blue, null);
			int hue = (int)(100.0 * hsb[0]);
			int saturation = (int)(100.0 * hsb[1]);
			int brightness = (int)(100.0 * hsb[2]);
			
			// Gadgets einrichten
			tabbedpane.setSelectedIndex(tabindex);
			colorfield.setBackground(color);
			colorfield.setPreferredSize(new Dimension(100,100));
			
			redslider.setValue(red);
			greenslider.setValue(green);
			blueslider.setValue(blue);
			hueslider.setValue(hue);
			saturationslider.setValue(saturation);
			brightnessslider.setValue(brightness);
			
			huetext.setEditable(false);
			saturationtext.setEditable(false);
			brightnesstext.setEditable(false);
			redtext.setEditable(false);
			greentext.setEditable(false);
			bluetext.setEditable(false);
			
			redtext.setText(""+red);
			greentext.setText(""+green);
			bluetext.setText(""+blue);
			huetext.setText(""+hue);
			saturationtext.setText(""+saturation);
			brightnesstext.setText(""+brightness);
			
			// Listener
			redslider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) { changedRed(); } });
			greenslider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) { changedGreen(); } });
			blueslider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) { changedBlue(); } });
			hueslider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) { changedHue(); } });
			saturationslider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) { changedSaturation(); } });
			brightnessslider.addChangeListener(new ChangeListener() { public void stateChanged(ChangeEvent e) { changedBrightness(); } });
			
			// Farben Panel erzeugen
			Box colorbox = Box.createHorizontalBox();
			colorbox.add(colorfield);
			colorbox.add(new Strut());
			colorbox.add(tabbedpane);
			
			// Buttons erzeugen
			OkCancelButtons buttons = new OkCancelButtons(ColorRequester.this,
				new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); } },
				new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); } },
				listener);
			
			// Panel erzeugen
			JPanel p = new YFramePanel();
			p.add(colorbox);
			p.add(new Strut());
			p.add(buttons);
			
			// Alles hinzufgen
			setContentPane(p);
			getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
			pack();
		}

		private void cancel()  {
			answer = CANCEL;
			tabindex = tabbedpane.getSelectedIndex();
			setVisible(false);
		}

		private void changedBlue()  {
			if (! handleEvent) return;
			
			// Events nicht mehr behandeln wegen setValue()
			handleEvent = false;
			
			// Werte auslesen
			int red = redslider.getValue();
			int green = greenslider.getValue();
			int blue = blueslider.getValue();
			
			// Farbe ndern
			Color c = new Color(red, green, blue);
			colorfield.setBackground(c);
			
			// Red Wert anzeigen
			bluetext.setText(""+blue);
			
			// HSB Werte ausrechnen
			float [] hsb = Color.RGBtoHSB(red, green, blue, null);
			int hue = (int)(100.0 * hsb[0]);
			int saturation = (int)(100.0 * hsb[1]);
			int brightness = (int)(100.0 * hsb[2]);
			
			// Slider fr HSB ndern
			hueslider.setValue(hue);
			saturationslider.setValue(saturation);
			brightnessslider.setValue(brightness);
			
			// Teste fr HSB ndern
			huetext.setText(""+hue);
			saturationtext.setText(""+saturation);
			brightnesstext.setText(""+brightness);
			
			// Events wieder behandeln
			handleEvent = true;
		}

		private void changedBrightness()  {
			if (! handleEvent) return;
			
			// Events nicht mehr behandeln wegen setValue()
			handleEvent = false;
			
			// Werte auslesen
			int hue = hueslider.getValue();
			int saturation = saturationslider.getValue();
			int brightness = brightnessslider.getValue();
			
			// Farbe ndern
			Color c = new Color(Color.HSBtoRGB((float)hue/100,(float)saturation/100,(float)brightness/100));
			colorfield.setBackground(c);
			
			// RGB Werte holen
			int red = c.getRed();
			int green = c.getGreen();
			int blue = c.getBlue();
			
			// Red Wert anzeigen
			brightnesstext.setText(""+brightness);
			
			// Slider fr RGB ndern
			redslider.setValue(red);
			greenslider.setValue(green);
			blueslider.setValue(blue);
			
			// Teste fr RGB ndern
			redtext.setText(""+red);
			greentext.setText(""+green);
			bluetext.setText(""+blue);
			
			// Events wieder behandeln
			handleEvent = true;
		}

		private void changedGreen()  {
			if (! handleEvent) return;
			
			// Events nicht mehr behandeln wegen setValue()
			handleEvent = false;
			
			// Werte auslesen
			int red = redslider.getValue();
			int green = greenslider.getValue();
			int blue = blueslider.getValue();
			
			// Farbe ndern
			Color c = new Color(red, green, blue);
			colorfield.setBackground(c);
			
			// Red Wert anzeigen
			greentext.setText(""+green);
			
			// HSB Werte ausrechnen
			float [] hsb = Color.RGBtoHSB(red, green, blue, null);
			int hue = (int)(100.0 * hsb[0]);
			int saturation = (int)(100.0 * hsb[1]);
			int brightness = (int)(100.0 * hsb[2]);
			
			// Slider fr HSB ndern
			hueslider.setValue(hue);
			saturationslider.setValue(saturation);
			brightnessslider.setValue(brightness);
			
			// Teste fr HSB ndern
			huetext.setText(""+hue);
			saturationtext.setText(""+saturation);
			brightnesstext.setText(""+brightness);
			
			// Events wieder behandeln
			handleEvent = true;
		}

		private void changedHue()  {
			if (! handleEvent) return;
			
			// Events nicht mehr behandeln wegen setValue()
			handleEvent = false;
			
			// Werte auslesen
			int hue = hueslider.getValue();
			int saturation = saturationslider.getValue();
			int brightness = brightnessslider.getValue();
			
			// Farbe ndern
			Color c = new Color(Color.HSBtoRGB((float)hue/100,(float)saturation/100,(float)brightness/100));
			colorfield.setBackground(c);
			
			// RGB Werte holen
			int red = c.getRed();
			int green = c.getGreen();
			int blue = c.getBlue();
			
			// Red Wert anzeigen
			huetext.setText(""+hue);
			
			// Slider fr RGB ndern
			redslider.setValue(red);
			greenslider.setValue(green);
			blueslider.setValue(blue);
			
			// Teste fr RGB ndern
			redtext.setText(""+red);
			greentext.setText(""+green);
			bluetext.setText(""+blue);
			
			// Events wieder behandeln
			handleEvent = true;
		}

		private void changedRed()  {
			if (! handleEvent) return;
			
			// Events nicht mehr behandeln wegen setValue()
			handleEvent = false;
			
			// Werte auslesen
			int red = redslider.getValue();
			int green = greenslider.getValue();
			int blue = blueslider.getValue();
			
			// Farbe ndern
			Color c = new Color(red, green, blue);
			colorfield.setBackground(c);
			
			// Red Wert anzeigen
			redtext.setText(""+red);
			
			// HSB Werte ausrechnen
			float [] hsb = Color.RGBtoHSB(red, green, blue, null);
			int hue = (int)(100.0 * hsb[0]);
			int saturation = (int)(100.0 * hsb[1]);
			int brightness = (int)(100.0 * hsb[2]);
			
			// Slider fr HSB ndern
			hueslider.setValue(hue);
			saturationslider.setValue(saturation);
			brightnessslider.setValue(brightness);
			
			// Teste fr HSB ndern
			huetext.setText(""+hue);
			saturationtext.setText(""+saturation);
			brightnesstext.setText(""+brightness);
			
			// Events wieder behandeln
			handleEvent = true;
		}

		private void changedSaturation()  {
			if (! handleEvent) return;
			
			// Events nicht mehr behandeln wegen setValue()
			handleEvent = false;
			
			// Werte auslesen
			int hue = hueslider.getValue();
			int saturation = saturationslider.getValue();
			int brightness = brightnessslider.getValue();
			
			// Farbe ndern
			Color c = new Color(Color.HSBtoRGB((float)hue/100,(float)saturation/100,(float)brightness/100));
			colorfield.setBackground(c);
			
			// RGB Werte holen
			int red = c.getRed();
			int green = c.getGreen();
			int blue = c.getBlue();
			
			// Red Wert anzeigen
			saturationtext.setText(""+saturation);
			
			// Slider fr RGB ndern
			redslider.setValue(red);
			greenslider.setValue(green);
			blueslider.setValue(blue);
			
			// Teste fr RGB ndern
			redtext.setText(""+red);
			greentext.setText(""+green);
			bluetext.setText(""+blue);
			
			// Events wieder behandeln
			handleEvent = true;
		}

		private JPanel createHSBPane()  {
			JPanel result = new JPanel();
			result.setLayout(new GridBagLayout());
			
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.BOTH;
			c.insets = JReqTools.getObjectInsets(-1,-1,-1,-1);
			c.weighty = 0;
			c.gridwidth = 1;
			c.gridheight = 1;
			
			c.gridx = 0; c.gridy = 0; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Hue")), c);
			c.gridx = 1; c.weightx = 1;
			result.add(hueslider, c);
			c.gridx = 2; c.weightx = 0;
			result.add(huetext, c);
			
			c.gridx = 0; c.gridy = 1; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Saturation")), c);
			c.gridx = 1; c.weightx = 1;
			result.add(saturationslider, c);
			c.gridx = 2; c.weightx = 0;
			result.add(saturationtext, c);
			
			c.gridx = 0; c.gridy = 2; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Brightness")), c);
			c.gridx = 1; c.weightx = 1;
			result.add(brightnessslider, c);
			c.gridx = 2; c.weightx = 0;
			result.add(brightnesstext, c);
			
			return result;
		}

		private JPanel createRGBPane()  {
			JPanel result = new JPanel();
			result.setLayout(new GridBagLayout());
			
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.BOTH;
			c.insets = JReqTools.getObjectInsets(-1,-1,-1,-1);
			c.weighty = 0;
			c.gridwidth = 1;
			c.gridheight = 1;
			
			c.gridx = 0; c.gridy = 0; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Red")), c);
			c.gridx = 1; c.weightx = 1;
			result.add(redslider, c);
			c.gridx = 2; c.weightx = 0;
			result.add(redtext, c);
			
			c.gridx = 0; c.gridy = 1; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Green")), c);
			c.gridx = 1; c.weightx = 1;
			result.add(greenslider, c);
			c.gridx = 2; c.weightx = 0;
			result.add(greentext, c);
			
			c.gridx = 0; c.gridy = 2; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Blue")), c);
			c.gridx = 1; c.weightx = 1;
			result.add(blueslider, c);
			c.gridx = 2; c.weightx = 0;
			result.add(bluetext, c);
			
			return result;
		}

		private void ok()  {
			answer = OK;
			tabindex = tabbedpane.getSelectedIndex();
			color = new Color(redslider.getValue(), greenslider.getValue(), blueslider.getValue());
			setVisible(false);
		}

	}

	/**
	* Constructs a new StringRequester.
	*/

	public ColorRequester()  {
		this("JReqTools: ColorRequester", Color.red);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public ColorRequester(String title)  {
		this(title, Color.red);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	defaultstring - The initial value of the text field.
	*/

	public ColorRequester(Color defaultcolor)  {
		this("JReqTools: ColorRequester",  defaultcolor);
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public ColorRequester(String title, Color defaultcolor)  {
		color = defaultcolor;
	}

	protected javax.swing.JDialog createJDialog(JFrame parent)  {
		return new Dialog(parent);
	}

	public Color getColor()  {
		return color;
	}

	public int getReturnValue()  {
		return answer;
	}

	public void setColor(Color newcolor)  {
		color = newcolor;
	}

}
