/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import javax.swing.*;
import java.io.*;

public class FileRequester {
	public static final int CANCEL		= -1;
	public static final int OK				= 0;
	
	JFileChooser 		chooser 		= new JFileChooser();
	boolean			loadmode		= true;
	boolean			dirsonly			= false;
	boolean			multiselect		= false;
	
	public FileRequester()  {
		this("JReqTools: FileRequester", new File(System.getProperty("user.home", "")), "");
	}

	public FileRequester(String title)  {
		this(title, new File(System.getProperty("user.home", "")), "");
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public FileRequester(String title, File directory, String filename)  {
		chooser.setDialogTitle(title);
		chooser.setSelectedFile(new File(directory, filename));
		chooser.setAcceptAllFileFilterUsed(false);
		chooser.addChoosableFileFilter(new FileFilter.AllFilesFileFilter());
	}

	static {
		UIManager.put("FileChooser.saveButtonText"				, JReqTools.getResourceString("FileChooser.Save"));
		UIManager.put("FileChooser.openButtonText"				, JReqTools.getResourceString("FileChooser.Open"));
		UIManager.put("FileChooser.cancelButtonText"			, JReqTools.getResourceString("FileChooser.Cancel"));
		UIManager.put("FileChooser.updateButtonText"			, JReqTools.getResourceString("FileChooser.Update"));
		UIManager.put("FileChooser.helpButtonText"				, JReqTools.getResourceString("FileChooser.Help"));
		
		UIManager.put("FileChooser.saveButtonToolTipText"		, JReqTools.getResourceString("FileChooser.SaveToolTip"));
		UIManager.put("FileChooser.openButtonToolTipText"		, JReqTools.getResourceString("FileChooser.OpenToolTip"));
		UIManager.put("FileChooser.cancelButtonToolTipText"		, JReqTools.getResourceString("FileChooser.CancelToolTip"));
		UIManager.put("FileChooser.updateButtonToolTipText"		, JReqTools.getResourceString("FileChooser.UpdateToolTip"));
		UIManager.put("FileChooser.helpButtonToolTipText"		, JReqTools.getResourceString("FileChooser.HelpToolTip"));
		
		UIManager.put("FileChooser.lookInLabelText"				, JReqTools.getResourceString("FileChooser.LookIn"));
		UIManager.put("FileChooser.fileNameLabelText"			, JReqTools.getResourceString("FileChooser.Filename"));
		UIManager.put("FileChooser.filesOfTypeLabelText"			, JReqTools.getResourceString("FileChooser.FilesOfType"));
		UIManager.put("FileChooser.filesLabelText"				, JReqTools.getResourceString("FileChooser.Files"));
		UIManager.put("FileChooser.foldersLabelText"				, JReqTools.getResourceString("FileChooser.Folders"));
		UIManager.put("FileChooser.pathLabelText"				, JReqTools.getResourceString("FileChooser.Path"));
		UIManager.put("FileChooser.filterLabelText"				, JReqTools.getResourceString("FileChooser.Filter"));
		UIManager.put("FileChooser.enterFileNameLabelText"		, JReqTools.getResourceString("FileChooser.EnterFileName"));
		
		UIManager.put("FileChooser.upFolderToolTipText"			, JReqTools.getResourceString("FileChooser.UpToolTip"));
		UIManager.put("FileChooser.homeFolderToolTipText"		, JReqTools.getResourceString("FileChooser.HomeToolTip"));
		UIManager.put("FileChooser.newFolderToolTipText"			, JReqTools.getResourceString("FileChooser.NewToolTip"));
		UIManager.put("FileChooser.listViewButtonToolTipText"		, JReqTools.getResourceString("FileChooser.ListViewToolTip"));
		UIManager.put("FileChooser.detailsViewButtonToolTipText"	, JReqTools.getResourceString("FileChooser.DetailsViewToolTip"));
	}

	public void addChoosableFileFilter(javax.swing.filechooser.FileFilter f)  {
		chooser.addChoosableFileFilter(f);
	}

	/**
	* @deprecated
	*/

	public void addChoosableFileFilter(javax.swing.filechooser.FileFilter f, boolean makeDefault)  {
		addChoosableFileFilter(f);
	}

	public File getFile()  {
		return chooser.getSelectedFile();
		
	}

	public File [] getFiles()  {
		return chooser.getSelectedFiles();
	}

	public int request()  {
		return this.request(null);
	}

	public int request(JFrame parent)  {
		int result = loadmode ? chooser.showOpenDialog(parent) : chooser.showSaveDialog(parent);
		if (result == JFileChooser.APPROVE_OPTION) return OK;
		return CANCEL;
	}

	public void setDirectoriesOnly(boolean bool)  {
		if (bool && multiselect)
			{
			System.err.println(getClass().getName() + ".setDirectoriesOnly(): multi selection and directory selection are not allowed together!");
			return;
			}
		
		dirsonly = bool;
		chooser.setFileSelectionMode(bool ? JFileChooser.DIRECTORIES_ONLY : JFileChooser.FILES_ONLY);
	}

	public void setFile(File file)  {
		chooser.setSelectedFile(file);
	}

	public void setFile(File directory, String filename)  {
		chooser.setSelectedFile(new File(directory, filename));
	}

	public void setLoadMode()  {
		loadmode = true;
	}

	public void setMultiSelection(boolean b)  {
		if (b && dirsonly)
			{
			System.err.println(getClass().getName() + ".setMultiSelection(): multi selection and directory selection are not allowed together!");
			return;
			}
		
		multiselect = b;
		chooser.setMultiSelectionEnabled(b);
	}

	public void setSaveMode()  {
		loadmode = false;
	}

	public void setTitle(String title)  {
		chooser.setDialogTitle(title);
	}

}
