/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.Dimension;
import java.awt.event.*;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import jreqtools.gadgets.*;
import jreqtools.utils.Sorter;

/**
* This is a requester for files or directories.
* 
* @author	Markus Hillenbrand
*/

public class FileRequester2 extends AbstractRequester {
	private 	File 		directory	= null;
	private	String		filename		= null;
	private 	String [] 	filenames	= null;
	private	boolean		loadmode	= true;
	private	boolean		dirsonly		= false;
	private	boolean		showdots	= false;
	private	int 			answer		= CANCEL;
	private	boolean		multiselect	= false;
	private	Vector		filefilters		= new Vector();
	private	int			filter			= 0;
	
	private class Dialog extends JDialog {
		private final String 	DIR 		= JReqTools.getResourceString("DIR") + " ";
		private final String	PARENT	= DIR  + JReqTools.getResourceString("ParentDirectory");
		
		private	File			directory	= FileRequester2.this.directory;
		private	TButton		home		= new TButton(JReqTools.getIcon("home"));
		private	TButton		reload		= new TButton(JReqTools.getIcon("reload"));
		private	TButton		mkdir		= new TButton(JReqTools.getIcon("makedir"));
		private	TButton		parent		= new TButton(JReqTools.getIcon("parent"));
		private 	TButton		toggledot	= new TButton(JReqTools.getIcon("toggledot"));
		
		private	JList		list			= new JList();
		private	TextField	directorytf	= new TextField("");
		private	TextField	filenametf	= new TextField("");
		private	JComboBox	filters		= new JComboBox(filefilters);
		private	JScrollPane	scrollpane	= new JScrollPane(list, 
												JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
												JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		private JPopupMenu	popupmenu	= new JPopupMenu();
		
		private class MyRenderer extends DefaultListCellRenderer {
			private final Icon directory	= JReqTools.getIcon("directory");
			private final Icon file			= JReqTools.getIcon("file");
			private final Icon parent		= JReqTools.getIcon("parent");
			private final Icon device		= JReqTools.getIcon("HardDrive");
			
			public java.awt.Component getListCellRendererComponent(javax.swing.JList l, java.lang.Object value, int index, boolean selected, boolean focus)  {
				String text = "";
				
				if (value instanceof String)
					{
					String s = (String) value;
					text = s.startsWith(DIR) ? s.substring(DIR.length()) : s;
					setIcon(s == PARENT ? parent : s.startsWith(DIR) ? directory : file);
					}
				else
					{
					setIcon(null);
					text = value.toString();
					}
							
				return super.getListCellRendererComponent(l, text, index, selected, focus);
			}

		}

		public Dialog(JFrame prnt)  {
			super(prnt, "JReqTools: FileRequester", true);
			
			// Gadgets einrichten
			scrollpane.setPreferredSize(new Dimension(300,400));
			directorytf.setText(directory.toString());
			filenametf.setText(filename);
			filters.setSelectedIndex(filter);
			list.setListData(getDirectory(directory));
			list.add(popupmenu);
			list.setCellRenderer(new MyRenderer());
			if (! multiselect) list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			list.setSelectedValue(filename, true);
			if (list.getSelectedIndex() == -1) list.setSelectedIndex(0);
			
			// PopupMenu
			initPopupMenu();
			
			// Listener
			list.addListSelectionListener(new ListSelectionListener() { 
				public void valueChanged(ListSelectionEvent e) { 
					selected(e); 
				}
			});
			list.addMouseListener(new MouseAdapter() { 
				public void mouseClicked(MouseEvent e) { 
					selected(e); 
				}
			});
			list.addKeyListener(new KeyAdapter() { 
				public void keyTyped(KeyEvent e) { 
					selected(e); 
				} 
			});
			directorytf.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					selected(e);
				}
			});
			filenametf.addActionListener(new ActionListener() { 
				public void actionPerformed(ActionEvent e) { 
					list.setSelectedValue(filenametf.getText(), true); 
					ok(); 
				} 
			});
			filters.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String [] s = getDirectory(directory);
					list.setListData(s);
					if (s.length > 0) list.setSelectedValue(s[0], true);
				}
			});
			home.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					directorytf.setText(System.getProperty("user.home"));
					selected(e);
				}
			});
			parent.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					parent();
				}
			});
			reload.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					reload();
				}
			});
			mkdir.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					makeDir();
					reload();
				}
			});
			toggledot.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					toggledot(e);
				}
			});
			
			// Obere Leiste erzeugen
			XPanel tools = new XPanel();
			tools.add(parent);
			tools.add(home);
			tools.add(reload);
			tools.add(mkdir);
			tools.add(toggledot);
			tools.add(Box.createHorizontalGlue());
			
			// Buttons erzeugen
			OkCancelButtons buttons = new OkCancelButtons(FileRequester2.this,
				new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); } },
				new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); } },
				listener);
			buttons.setButtonText(0, loadmode ? JReqTools.getResourceString("Load") : JReqTools.getResourceString("Save"));
			if (dirsonly) buttons.setButtonText(0, JReqTools.getResourceString("Select"));
			
			// Panel erzeugen
			JPanel p = new YFramePanel();
			p.add(tools);
			p.add(scrollpane);
			p.add(new Strut());
			p.add(createPanel());
			p.add(new Strut());
			p.add(buttons);
			
			// Alles hinzufgen
			setContentPane(p);
			getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
			pack();
		}

		private void cancel()  {
			answer = CANCEL;
			setVisible(false);
		}

		private Panel createPanel()  {
			Panel result = new Panel() {
				public Dimension getMaximumSize() { 
					return new Dimension(Short.MAX_VALUE, getPreferredSize().height);
				}
			};
			
			result.setLayout(new GridBagLayout());
			
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.BOTH;
			c.weighty = 0;
			c.gridwidth = 1;
			c.gridheight = 1;
			
			if (! dirsonly)
				{
				c.insets = JReqTools.getObjectInsets(0,0,-1,-1); c.gridx = 0; c.gridy = 0; c.weightx = 0;
				result.add(new JLabel(JReqTools.getResourceString("FileFilter")), c);
				c.insets = JReqTools.getObjectInsets(0,0,-1,0); c.gridx = 1; c.weightx = 1;
				result.add(filters, c);
				}
			
			c.insets = JReqTools.getObjectInsets(0,0,-1,-1); c.gridx = 0; c.gridy = 1; c.weightx = 0;
			result.add(new JLabel(JReqTools.getResourceString("Directory")), c);
			c.insets = JReqTools.getObjectInsets(0,0,-1,0); c.gridx = 1; c.weightx = 1;
			result.add(directorytf, c);
			
			if (! dirsonly && ! multiselect)
				{
				c.insets = JReqTools.getObjectInsets(0,0,0,-1); c.gridx = 0; c.gridy = 2; c.weightx = 0;
				result.add(new JLabel(JReqTools.getResourceString("Filename")), c);
				c.insets = JReqTools.getObjectInsets(0,0,0,0); c.gridx = 1; c.weightx = 1; c.gridwidth=3;
				result.add(filenametf, c);
				}
			
			return result;
		}

		private String [] getDirectory(File dir)  {
			String [] directories 	= dir.list(new DirectoryFilenameFilter());
			String [] files		= dirsonly ? null : dir.list(new FileFilenameFilter((FileFilter)filefilters.elementAt(filters.getSelectedIndex())));
			
			int count = 0;
			if (directories != null) 
				{
				Sorter.sort(directories);
				count = directories.length;
				}
			if (files != null) 
				{
				Sorter.sort(files);
				count += files.length;
				}
			
			String [] result = new String[count+1];
			
			result[0] = PARENT;
			if (directories != null) for (int i=0; i<directories.length; ++i) result[i+1] = DIR + directories[i];
			if (files != null) for (int i=0; i<files.length; ++i) result[i+directories.length+1] = files[i];
			
			return result;
		}

		private void initPopupMenu()  {
			File file = new File("/");
			if (! file.exists()) return;
			
			JMenuItem item = new JMenuItem("Root Directory");
			item.addActionListener(new ActionListener() { 
				public void actionPerformed(ActionEvent e) { 
					directorytf.setText("/"); 
					selected(e); 
				}
			});
			popupmenu.add(item);
			
			item = new JMenuItem("Home Directory");
			item.addActionListener(new ActionListener() { 
				public void actionPerformed(ActionEvent e) { 
					directorytf.setText(System.getProperty("user.home")); 
					selected(e); 
				}
			});
			popupmenu.add(item);
			
			file = new File(System.getProperty("user.home"), "Desktop");
			if (file.exists() && file.isDirectory())
				{
				item = new JMenuItem("KDE Desktop Directory");
				item.addActionListener(new ActionListener() { 
					public void actionPerformed(ActionEvent e) { 
						directorytf.setText(System.getProperty("user.home") + File.separator + "Desktop"); 
						selected(e); 
					}
				});
				popupmenu.add(item);
				}
			
			list.add(popupmenu);
			list.addMouseListener(new MouseAdapter() {
				private void doit(MouseEvent e) {
					if (e.isPopupTrigger()) 
						{
						popupmenu.show(list, e.getX()-5, e.getY()-5);
						e.consume();
						}
				}
				public void mousePressed(MouseEvent e) { doit(e); }
				public void mouseReleased(MouseEvent e) { doit(e); }
			});
		}

		private void makeDir()  {
			String s = JReqTools.requestString(null,
				JReqTools.getResourceString("CreateDirectoryTitle"), 
				JReqTools.getResourceString("CreateDirectoryMessage"),
				"");
				
			if (s.length() == 0) return;
			
			File f = new File(directory, s);
			if (f.mkdir()) return;
			
			JReqTools.showError(JReqTools.getResourceString("DirectoryNotCreated"));
		}

		private void ok()  {
			if (multiselect)
				{
				// Objekte holen
				Object [] o = list.getSelectedValues();
				if (o == null)
					{
					JReqTools.showError(JReqTools.getResourceString("NoFileName"));
					return;
					}
				// Dateien zhlen
				Vector v = new Vector();
				for (int i=0; i<o.length; ++i) if (o[i] != PARENT && ! ((String)o[i]).startsWith(DIR)) v.addElement(o[i]);
				if (v.size() == 0)
					{
					JReqTools.showError(JReqTools.getResourceString("NoFileName"));
					return;
					}
				// Dateinamen eintragen
				FileRequester2.this.filenames = new String[v.size()];
				for (int i=0; i<v.size(); ++i) FileRequester2.this.filenames[i] = (String) v.elementAt(i);
				// Und fertig
				FileRequester2.this.answer 	= OK;
				FileRequester2.this.directory	= directory;
				setVisible(false);
				return;
				}
			
			if (dirsonly)
				{
				FileRequester2.this.answer 	= OK;
				FileRequester2.this.directory	= directory;
				setVisible(false);
				return;
				}
			
			if (filenametf.getText().length() == 0)
				{
				getToolkit().beep();
				JReqTools.showError(JReqTools.getResourceString("NoFileName"));
				filenametf.requestFocus();
				return;
				}
			
			File f = new File(directory, filenametf.getText());
			if (f.isDirectory())
				{
				getToolkit().beep();
				JReqTools.showError("'" + f.toString() + "'\n" + JReqTools.getResourceString("IsADirectory"));
				filenametf.requestFocus();
				return;
				}
			
			f = new File(directory, filenametf.getText());
			if (loadmode && ! f.exists())
				{
				getToolkit().beep();
				JReqTools.showError("'" + f.toString() + "'\n" + JReqTools.getResourceString("DoesNotExist"));
				filenametf.requestFocus();
				return;
				}
			
			FileRequester2.this.answer 	= OK;
			FileRequester2.this.directory	= directory;
			FileRequester2.this.filename	= filenametf.getText();
			FileRequester2.this.filter		= filters.getSelectedIndex();
			
			setVisible(false);
		}

		private void parent()  {
			String parent = directory != null ? directory.getParent() : null;
			if (parent == null)
				{
				File [] roots = File.listRoots();
				if (directory != null && roots != null && roots.length > 1)
					{
					String [] s = new String[roots.length];
					for (int i=0; i<s.length; ++i) s[i] = DIR + roots[i].toString();
					list.setListData(s);
					directory = null;
					directorytf.setText("");
					list.setSelectedValue(s[0], true);
					return;
					}
				getToolkit().beep();
				JReqTools.showError(JReqTools.getResourceString("ParentDoesNotExist"));
				return;
				}
			
			directory = new File(parent);
			
			String [] s = getDirectory(directory);
			list.setListData(s);
			directorytf.setText(directory.toString());
			if (s.length > 0) list.setSelectedValue(s[0], true);
		}

		private void reload()  {
			String [] s = getDirectory(directory);
			list.setListData(s);
			if (s.length > 0) list.setSelectedValue(s[0], true);
		}

		private void selected(ListSelectionEvent e)  {
			if (e.getValueIsAdjusting()) return;
			
			String selected = (String) list.getSelectedValue();
			if (selected == null) return;
			
			if (selected.startsWith(DIR)) return;
			
			filenametf.setText(selected);
		}

		private void selected(ActionEvent e)  {
			File f = new File(directorytf.getText());
			if (! f.isDirectory()) 
				{
				getToolkit().beep();
				JReqTools.showError("'" + f.toString() + "'\n" + JReqTools.getResourceString("DoesNotExist2"));
				return;
				}
			
			String [] s = getDirectory(f);
			list.setListData(s);
			if (s.length > 0) list.setSelectedValue(s[0], true);
			
			directory = f;
		}

		private void selected(KeyEvent e)  {
			if (e.getKeyChar() == KeyEvent.VK_ESCAPE)
				{
				cancel();
				return;
				}
			
			if (e.getKeyChar() != KeyEvent.VK_ENTER) return;
			
			String selected = (String) list.getSelectedValue();
			if (selected.startsWith(DIR) || selected == PARENT) 
				{
				if (selected == PARENT)
					{
					parent();
					return;
					}
				directory = directory != null ?  new File(directory, selected.substring(DIR.length())):  new File(selected.substring(DIR.length()));
				String [] s = getDirectory(directory);
				list.setListData(s);
				directorytf.setText(directory.toString());
				if (s.length > 0) list.setSelectedValue(s[0], true);
				}
			else 
				{
				filenametf.setText(selected);
				ok();
				}
		}

		private void selected(MouseEvent e)  {
			String selected = (String) list.getSelectedValue();
			if (selected.startsWith(DIR) || selected == PARENT) 
				{
				if (e.getClickCount() < 2) return;
				if (selected == PARENT)
					{
					parent();
					return;
					}
				directory = directory != null ? new File(directory, selected.substring(DIR.length())) : new File(selected.substring(DIR.length()));
				String [] s = getDirectory(directory);
				list.setListData(s);
				directorytf.setText(directory.toString());
				if (s.length > 0) list.setSelectedValue(s[0], false);
				}
			else 
				{
				filenametf.setText(selected);
				if (e.getClickCount() > 1) ok();
				}
			
		}

		public void setVisible(boolean bool)  {
			if (bool) list.requestFocus();
			super.setVisible(bool);
		}

		public void toggledot(ActionEvent e)  {
			FileRequester2.this.showdots = ! FileRequester2.this.showdots;
			reload();
		}

	}

	private class DirectoryFilenameFilter implements FilenameFilter {
		
		public boolean accept(java.io.File arg0, java.lang.String arg1)  {
			if (! showdots)
				{
				if (arg1.charAt(0) == '.') return false;
				}
			
			return new File(arg0, arg1).isDirectory();
		}

	}

	private class FileFilenameFilter implements FilenameFilter {
		FileFilter filefilter;
		
		public FileFilenameFilter(FileFilter f)  {
			filefilter = f;
		}

		public boolean accept(java.io.File arg0, java.lang.String arg1)  {
			if (arg1.charAt(0) == '.') return false;
			File f = new File(arg0, arg1);
			if (f.isDirectory()) return false;
			return filefilter.accept(f);
		}

	}

	public FileRequester2()  {
		this("JReqTools: FileRequester", new File(System.getProperty("user.home", "")), "");
	}

	public FileRequester2(String title)  {
		this(title, new File(System.getProperty("user.home", "")), "");
	}

	/**
	* Constructs a new StringRequester.
	* 
	* @param	title - Title of the requester.
	* @param	message - The message to be shown in the requester.
	* @param	defaultstring - The initial value of the text field.
	*/

	public FileRequester2(String title, File directory, String filename)  {
		setTitle(title);
		setFile(directory, filename);
		//addChoosableFileFilter(new FileFilter.AllFilesFileFilter(), true);
	}

	public void addChoosableFileFilter(FileFilter f)  {
		addChoosableFileFilter(f, false);
	}

	public void addChoosableFileFilter(FileFilter f, boolean makeDefault)  {
		if (filefilters.contains(f)) return;
		filefilters.addElement(f);
		if (makeDefault) filter = filefilters.indexOf(f);
		
	}

	protected javax.swing.JDialog createJDialog(JFrame parent)  {
		return new Dialog(parent);
	}

	public File getFile()  {
		return dirsonly ? directory : new File(directory, multiselect ? "" : filename);
	}

	public File [] getFiles()  {
		if (! multiselect) 
			{
			JReqTools.showError(JReqTools.getResourceString("MethodCallNotAllowed") + "\n\nFileRequester.getFile()");
			return null;
			}
		
		File [] result = new File[filenames.length];
		for (int i=0; i<filenames.length; ++i) result[i] = new File(directory, filenames[i]);
		
		return result;
	}

	protected int getReturnValue()  {
		return answer;
	}

	public void setDirectoriesOnly(boolean bool)  {
		if (bool && multiselect)
			{
			System.err.println(getClass().getName() + ".setDirectoriesOnly(): multi selection and directory selection are not allowed together!");
			return;
			}
		
		dirsonly = bool;
	}

	public void setFile(File f)  {
		if (f == null)
			{
			setFile(null, "");
			}
		else if (f.isDirectory())
			{
			setFile(f, "");
			}
		else
			{
			File dir = new File(f.getParent());
			String name = f.getName();
			setFile(dir, name);
			}
	}

	public void setFile(String filename)  {
		setFile(new File(filename));
	}

	public void setFile(File dir, String fname)  {
		if (dir != null && dir.exists() && dir.isDirectory())
			{
			directory = dir;
			filename = fname != null ? fname : "";
			}
		else
			{
			directory = new File(System.getProperty("user.home", ""));
			filename = fname != null ? fname : "";
			}
	}

	public void setLoadMode()  {
		loadmode = true;
	}

	public void setMultiSelection(boolean b)  {
		if (b && dirsonly)
			{
			System.err.println(getClass().getName() + ".setMultiSelection(): multi selection and directory selection are not allowed together!");
			return;
			}
		
		multiselect = b;
	}

	public void setSaveMode()  {
		loadmode = false;
	}

}
