/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import java.awt.Toolkit;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import jreqtools.gadgets.*;

public class TipOfTheDay extends JFrame {
	String [] 		tips			= null;
	int 				lastTip		= 0;
	JLabel			icon		= new JLabel(JReqTools.getIcon("tipOfTheDay"));
	JButton			next		= new JButton(JReqTools.getIcon("arrowRight"));
	JButton			prev		= new JButton(JReqTools.getIcon("arrowLeft"));
	JButton			close		= new JButton(JReqTools.getResourceString("CloseTip"), UIManager.getIcon("jreqtools.cancelIcon"));
	CheckBox		show		= new CheckBox(JReqTools.getResourceString("ShowNextTime"), true);
	TextField		t1			= new TextField("");
	TextField		t2			= new TextField("");
	TextField		t3			= new TextField("");
	TextField		t4			= new TextField("");
	TextField		t5			= new TextField("");
	JLabel			t6			= new JLabel("0/0");
	
	public TipOfTheDay(String title, String [] tips, int firstTipToShow, boolean showNextTime)  {
		super(title);
		
		// Initialisieren
		this.tips = tips;
		this.lastTip = firstTipToShow;
		this.show.setSelected(showNextTime);
		t1.setOpaque(false);
		t2.setOpaque(false);
		t3.setOpaque(false);
		t4.setOpaque(false);
		t5.setOpaque(false);
		t6.setOpaque(false);
		t1.setFont(show.getFont());
		t2.setFont(show.getFont());
		t3.setFont(show.getFont());
		t4.setFont(show.getFont());
		t5.setFont(show.getFont());
		t6.setFont(show.getFont());
		t1.setBorder(BorderFactory.createEmptyBorder());
		t2.setBorder(BorderFactory.createEmptyBorder());
		t3.setBorder(BorderFactory.createEmptyBorder());
		t4.setBorder(BorderFactory.createEmptyBorder());
		t5.setBorder(BorderFactory.createEmptyBorder());
		t6.setBorder(BorderFactory.createEmptyBorder());
		t1.setEditable(false);
		t2.setEditable(false);
		t3.setEditable(false);
		t4.setEditable(false);
		t5.setEditable(false);
		t6.setForeground(t5.getForeground());
		
		// Listener
		next.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lastTip = (lastTip + 1) % TipOfTheDay.this.tips.length;
				setTip(lastTip);
			}
		});
		prev.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				lastTip = lastTip-1;
				if (lastTip == -1) lastTip = TipOfTheDay.this.tips.length-1;
				setTip(lastTip);
			}
		});
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});
		
		// TextPanel
		int space = JReqTools.getFrameSpace();
		JPanel textPanel = new JPanel();
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.Y_AXIS));
		textPanel.add(t1);
		textPanel.add(t2);
		textPanel.add(t3);
		textPanel.add(t4);
		textPanel.add(t5);
		
		// So ?
		//textPanel.setOpaque(false);
		//textPanel.setBorder(BorderFactory.createEmptyBorder(space, space, space, space));
		// Oder so
		Color c = getBackground();
		float [] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
		textPanel.setBackground(new Color(java.awt.Color.HSBtoRGB(hsb[0], hsb[1], (float)Math.max(0.0, hsb[2]-0.1))));
		textPanel.setBorder(new BigBorder());
		
		// Oberes Panel
		XPanel topPanel = new XPanel();
		topPanel.add(textPanel);
		topPanel.add(new Strut());
		topPanel.add(icon);
		
		// Buttons
		XPanel buttons = new XPanel();
		buttons.add(prev);
		buttons.add(new Strut());
		buttons.add(t6);
		buttons.add(new Strut());
		buttons.add(next);
		buttons.add(new Strut());
		buttons.add(Box.createHorizontalGlue());
		buttons.add(close);
		
		// ShowPanel
		XPanel showPanel = new XPanel();
		showPanel.add(show);
		showPanel.add(Box.createHorizontalGlue());
		
		// Panel erzeugen
		YFramePanel panel = new YFramePanel();
		panel.add(topPanel);
		panel.add(new Strut());
		panel.add(Box.createVerticalGlue());
		//panel.add(new Separator());
		//panel.add(new Strut());
		panel.add(showPanel);
		panel.add(new Strut());
		panel.add(buttons);
		
		// Alles hinzufgen
		setContentPane(panel);
		getLayeredPane().add(new BackgroundImage(), new Integer(Integer.MIN_VALUE));
		setTip(firstTipToShow);
		
		// Positionieren
		Toolkit tk = getToolkit();
		Dimension ds = tk.getScreenSize();
		Dimension dd = getSize();
		setLocation((ds.width - dd.width) / 2, (ds.height - dd.height) / 2 );
	}

	public boolean getShowNextTime()  {
		return show.isSelected();
	}

	public int getTip()  {
		return lastTip;
	}

	public void setIcon(Icon icon)  {
		this.icon.setIcon(icon);
	}

	public void setShowNextTime(boolean b)  {
		show.setSelected(b);
	}

	public void setTip(int i)  {
		lastTip = i;
		if (i >= tips.length) i=0;
		
		StringTokenizer tok = new StringTokenizer(tips[i], "\n");
		
		t1.setText(tok.hasMoreElements() ? tok.nextToken() : "");
		t2.setText(tok.hasMoreElements() ? tok.nextToken() : "");
		t3.setText(tok.hasMoreElements() ? tok.nextToken() : "");
		t4.setText(tok.hasMoreElements() ? tok.nextToken() : "");
		t5.setText(tok.hasMoreElements() ? tok.nextToken() : "");
		
		t6.setText("" + (i+1) + "/" + tips.length);
		
		pack();
	}

}
