/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jreqtools.test;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import jreqtools.*;

public class Main extends JFrame implements ActionListener {
	// Resources	
	private static final ResourceBundle resources = ResourceBundle.getBundle("jreqtools.test.resources.test");
	
	// ColorRequester
	ColorRequester 		colorRequester			= new ColorRequester(getResourceString("SelectColor"));
	JButton					colorRequesterB		= new JButton(getResourceString("Color"));
	// FontRequester
	FontRequester 			fontRequester			= new FontRequester(getResourceString("SelectFont"));
	JButton					fontRequesterB			= new JButton(getResourceString("Font"));
	// FileRequester
	FileRequester 			fileRequester			= new FileRequester(getResourceString("SelectFile"));
	JButton					fileRequesterB			= new JButton(getResourceString("File"));
	// ByteRequester
	ByteRequester 			byteRequester			= new ByteRequester(getResourceString("EnterByte"), Byte.MAX_VALUE);
	JButton					byteRequesterB		= new JButton(getResourceString("Byte"));
	// ShortRequester
	ShortRequester 		shortRequester			= new ShortRequester(getResourceString("EnterShort"), Short.MAX_VALUE);
	JButton					shortRequesterB		= new JButton(getResourceString("Short"));
	// IntegerRequester
	IntegerRequester 		integerRequester		= new IntegerRequester(getResourceString("EnterInteger"), Integer.MAX_VALUE);
	JButton					integerRequesterB		= new JButton(getResourceString("Integer"));
	// LongRequester
	LongRequester 		longRequester			= new LongRequester(getResourceString("EnterLong"), Long.MAX_VALUE);
	JButton					longRequesterB		= new JButton(getResourceString("Long"));
	// LongRequester
	FloatRequester 		floatRequester			= new FloatRequester(getResourceString("EnterFloat"), Float.MAX_VALUE);
	JButton					floatRequesterB		= new JButton(getResourceString("Float"));
	// LongRequester
	DoubleRequester 		doubleRequester		= new DoubleRequester(getResourceString("EnterDouble"), Double.MAX_VALUE);
	JButton					doubleRequesterB		= new JButton(getResourceString("Double"));
	// StringRequester
	StringRequester 		stringRequester		= new StringRequester(getResourceString("EnterString"), "");
	JButton					stringRequesterB		= new JButton(getResourceString("String"));
	// TextRequester
	TextRequester 			textRequester			= new TextRequester(getResourceString("EnterText") , "");
	JButton					textRequesterB			= new JButton(getResourceString("Text"));
	// PasswordRequester
	PasswordRequester 	passwordRequester	= new PasswordRequester(getResourceString("EnterPassword"));
	JButton					passwordRequesterB	= new JButton(getResourceString("Password"));
	// LoginRequester
	LoginRequester 		loginRequester			= new LoginRequester(getResourceString("LoginM"), 
														getResourceString("Login1"), getResourceString("Login2"));
	JButton					loginRequesterB		= new JButton(getResourceString("Login"));
	// PasswordRequester
	SelectRequester	 	selectRequester		= new SelectRequester(getResourceString("PleaseSelect"), 
														new String [] { getResourceString("Item1"), getResourceString("Item2"), 
														getResourceString("Item3") });
	JButton					selectRequesterB		= new JButton(getResourceString("Selection"));
	// LookAndFeelRequester
	LookAndFeelRequester	lookandfeelRequester	= new LookAndFeelRequester(getResourceString("PleaseSelectLookAndFeel"));
	JButton					lookandfeelRequesterB	= new JButton(getResourceString("LookAndFeel"));
	// ProgressTracker
	JButton					progressTrackerB		= new JButton(getResourceString("ProgressTracker"));
	// Tips
	TipOfTheDay			tipOfTheDay			= new TipOfTheDay(getResourceString("Tip.Title"), 
														new String [] { getResourceString("Tip.1"), getResourceString("Tip.2"), 
														getResourceString("Tip.3"),}, 0, true);
	
	private class Task extends ProgressTask {
		int max = 500;
		int current = 0;
		boolean cancelled = false;
		
		public void cancel()  {
			cancelled = true;
		}

		public int getCurrent()  {
			return current;
		}

		public java.lang.String getCurrentText()  {
			return getResourceString("Loading") + " " + current + "/" + max;
		}

		public int getMaximum()  {
			return max;
		}

		public int getMinimum()  {
			return 0;
		}

		public void run()  {
			for (int i=0; i<=max && ! cancelled; ++i)
				{
				current = i;
				for (int j=0; j<2000000; ++j); // Do something here :-)
				}
			
			// If you want the ProgressTracker to fill the JProgressBar 100%, then
			// you'll have to wait a bit more then 1 second (this is the worst case).
			
			try { Thread.sleep(1050); } catch(InterruptedException x) {}
		}

	}

	public Main()  {
		super ("JReqTools");
		
		// Gadgets einrichten
		stringRequester.addHelpButton(this);
		textRequester.addHelpButton(this);
		passwordRequester.addHelpButton(this);
		lookandfeelRequester.setAfterSelection(this);
		fileRequester.addChoosableFileFilter(new FileFilter.SuffixFileFilter(".class", getResourceString("JavaClassFiles")));
		fileRequester.addChoosableFileFilter(new FileFilter.SuffixFileFilter(".txt", getResourceString("TextFiles")));
		fileRequester.addChoosableFileFilter(new FileFilter.SuffixFileFilter(".java", getResourceString("JavaSourceFiles")));
		byteRequester.setDefaultChoices(new Object [] { new Byte(Byte.MIN_VALUE), new Byte(Byte.MAX_VALUE), new Byte((byte)0) });
		// You might do this for a multi selection file requester:
		//fileRequester.setMultiSelection(true);
		
		// Listeners
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				if (JReqTools.showQuestion(getResourceString("AskQuit"))) System.exit(0);
			}
		});
		byteRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (byteRequester.request(Main.this) != ByteRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" + byteRequester.getByte() + "'"); 
				}
		});
		shortRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (shortRequester.request(Main.this) != ShortRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" + shortRequester.getShort() + "'"); 
				}
		});
		integerRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (integerRequester.request(Main.this) != IntegerRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" + integerRequester.getInteger() + "'"); 
				}
		});
		longRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (longRequester.request(Main.this) != LongRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" + longRequester.getLong() + "'"); 
				}
		});
		floatRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (floatRequester.request(Main.this) != FloatRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" + floatRequester.getFloat() + "'"); 
				}
		});
		doubleRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (doubleRequester.request(Main.this) != DoubleRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" + doubleRequester.getDouble() + "'"); 
				}
		});
		stringRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (stringRequester.request(Main.this) != StringRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" +stringRequester.getString() + "'"); 
				}
		});
		textRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (textRequester.request(Main.this) != TextRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" +textRequester.getText() + "'"); 
				}
		});
		passwordRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (passwordRequester.request(Main.this) != PasswordRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered") +" '" +passwordRequester.getPassword() + "'"); 
			}
		});
		loginRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (loginRequester.request(Main.this) != LoginRequester.CANCEL) 
					System.out.println(getResourceString("YouEntered")+" '"+ loginRequester.getLogin()+"' and '"+loginRequester.getPassword()+"'");
			}
		});
		selectRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (selectRequester.request(Main.this) != SelectRequester.CANCEL) 
					System.out.println(getResourceString("YouSelected") +" '" + selectRequester.getSelectedValue() + "'"); 
			}
		});
		colorRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (colorRequester.request(Main.this) != ColorRequester.CANCEL) 
					System.out.println(getResourceString("YouSelected") +" '" + colorRequester.getColor() + "'"); 
			}
		});
		fontRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (fontRequester.request(Main.this) != FontRequester.CANCEL) 
					System.out.println(getResourceString("YouSelected") +" '" + fontRequester.getFont() + "'"); 
			}
		});
		fileRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (fileRequester.request(Main.this) != FileRequester.CANCEL) 
					{
					System.out.println(getResourceString("YouSelected") +" '" + fileRequester.getFile() + "'"); 
					// Or you do this for a multiselection file requester
					// File [] f = fileRequester.getFiles();
					// for (int i=0; i<f.length; ++i) System.out.println(getResourceString("YouSelected") +" '" + f[i] + "'"); 
					}
			}
		});
		lookandfeelRequesterB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (lookandfeelRequester.request(Main.this) == LookAndFeelRequester.CANCEL)  return;
				System.out.println(getResourceString("YouSelected") +" '" + lookandfeelRequester.getLookAndFeel() + "'"); 
				SwingUtilities.updateComponentTreeUI(SwingUtilities.getRoot(tipOfTheDay));
				pack();
			}
		});
		progressTrackerB.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ProgressTracker m = new ProgressTracker(getResourceString("ProgressTrackerMessage"), new Task());
				m.addCancelButton(true);
				m.start();
			}
		});
		
		// Create Choice Panel
		JPanel p = new JPanel();
		p.setOpaque(false);
		p.setLayout(new GridLayout(4,0,5,5));
		p.add(byteRequesterB);
		p.add(shortRequesterB);
		p.add(integerRequesterB);
		p.add(longRequesterB);
		p.add(floatRequesterB);
		p.add(doubleRequesterB);
		p.add(passwordRequesterB);
		p.add(loginRequesterB);
		p.add(stringRequesterB);
		p.add(textRequesterB);
		p.add(selectRequesterB);
		p.add(colorRequesterB);
		p.add(fontRequesterB);
		p.add(fileRequesterB);
		p.add(lookandfeelRequesterB);
		p.add(progressTrackerB);
		
		// Create Main Panel
		JPanel panel = new JPanel();
		panel.setOpaque(false);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		panel.add(p);
		
		// Menu 1
		JMenuBar menubar = new JMenuBar();
		JMenu menu = new JMenu("JReqTools");
		menubar.add(menu);
		JMenuItem item = new JMenuItem(getResourceString("About"));
		item.addActionListener(JReqTools.getAboutAction(this, getResourceString("AboutText")));
		item.setAccelerator(KeyStroke.getKeyStroke(getResourceString("VK_About")));
		menu.add(item);
		item = new JMenuItem(getResourceString("Tip"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { tipOfTheDay.setVisible(true); }});
		item.setAccelerator(KeyStroke.getKeyStroke(getResourceString("VK_Tip")));
		menu.add(item);
		menu.addSeparator();
		item = new JMenuItem(getResourceString("Quit"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { if (JReqTools.showQuestion(getResourceString("AskQuit"))) System.exit(0); }
			});
		item.setAccelerator(KeyStroke.getKeyStroke(getResourceString("VK_Quit")));
		menu.add(item);
		
		// Menu 2
		menu = menubar.add(new JMenu(getResourceString("Misc")));
		item = menu.add(new JMenuItem("JReqTools.showError()"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { JReqTools.showError(Main.this, getResourceString("Misc.Error")); }
			});
		item = menu.add(new JMenuItem("JReqTools.showException()"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { JReqTools.showException(Main.this, new IllegalArgumentException()); }
			});
		item = menu.add(new JMenuItem("JReqTools.showMessage()"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { JReqTools.showMessage(Main.this, getResourceString("Misc.Message")); }
			});
		item = menu.add(new JMenuItem("JReqTools.showNotYetImplemented()"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { JReqTools.showNotYetImplemented(Main.this); }
			});
		item = menu.add(new JMenuItem("JReqTools.showQuestion()"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { JReqTools.showQuestion(Main.this, getResourceString("Misc.Question"));  }
			});
		
		// Add everything
		setContentPane(panel);
		getLayeredPane().add(new jreqtools.gadgets.BackgroundImage(), new Integer(Integer.MIN_VALUE));
		setJMenuBar(menubar);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setLocation(100,100);
		pack();
		setVisible(true);
		tipOfTheDay.setVisible(true);
	}

	public void actionPerformed(java.awt.event.ActionEvent arg0)  {
		System.out.println(getResourceString("HelpRequested") + " '" + arg0.getSource() + "'");
	}

	private static String getResourceString(String text)  {
		try
			{
			return resources.getString(text);
			}
		catch (MissingResourceException x) 
			{
			JReqTools.showMessage(x.getLocalizedMessage() + ":\n\n'" + text + "'" );
			return "MISSING RESOURCE: " + text;
			}
	}

	public static void main(String [] args)  {
		UIManager.put("jreqtools.frameSpace"			, new Integer(12));				// Default is 10
		UIManager.put("jreqtools.objectSpace"		, new Integer(6));				// Default is 5
		UIManager.put("jreqtools.okIcon"				, JReqTools.getIcon("ok"));		// Default is null
		UIManager.put("jreqtools.cancelIcon"			, JReqTools.getIcon("cancel"));	// Default is null
		UIManager.put("jreqtools.helpIcon"			, JReqTools.getIcon("help"));	// Default is null
		
		// Uncomment the following line if you want a background pattern
		// UIManager.put("jreqtools.patternIcon"		, JReqTools.getIcon("pattern"));	// Default is null
		
		new Main();
	}

}
